@@ -257,6 +257,11 @@ all_errors = [
     ('no_such_attribute', "Object has no attribute 'wrong_name'", {'attribute': 'wrong_name'}),
     ('json_invalid', 'Invalid JSON: foobar', {'error': 'foobar'}),
     ('json_type', 'JSON input should be string, bytes or bytearray', None),
+    (
+        'needs_python_object',
+        'Cannot check `isinstance` when validating from json, use a JsonOrPython validator instead',
+        {'method_name': 'isinstance'},
+    ),
     ('recursion_loop', 'Recursion error - cyclic reference detected', None),
     ('model_type', 'Input should be a valid dictionary or instance of Foobar', {'class_name': 'Foobar'}),
     ('model_attributes_type', 'Input should be a valid dictionary or object to extract fields from', None),
@@ -301,6 +306,7 @@ all_errors = [
     ('iteration_error', 'Error iterating over object, error: foobar', {'error': 'foobar'}),
     ('list_type', 'Input should be a valid list', None),
     ('tuple_type', 'Input should be a valid tuple', None),
+    ('set_item_not_hashable', 'Set items should be hashable', None),
     ('set_type', 'Input should be a valid set', None),
     ('bool_type', 'Input should be a valid boolean', None),
     ('bool_parsing', 'Input should be a valid boolean, unable to interpret input', None),
@@ -506,10 +512,10 @@ def test_all_errors():
             'example_context': None,
         },
         {
-            'type': 'recursion_loop',
-            'message_template_python': 'Recursion error - cyclic reference detected',
-            'example_message_python': 'Recursion error - cyclic reference detected',
-            'example_context': None,
+            'type': 'needs_python_object',
+            'message_template_python': 'Cannot check `{method_name}` when validating from json, use a JsonOrPython validator instead',
+            'example_message_python': 'Cannot check `` when validating from json, use a JsonOrPython validator instead',
+            'example_context': {'method_name': ''},
         },
     ]
 
@@ -533,7 +539,7 @@ def test_all_errors():
 
 @pytest.mark.skipif(sys.version_info < (3, 11), reason='This is the modern version used post 3.10.')
 def test_validation_error_cause_contents():
-    enabled_config: CoreConfig = {'validation_error_cause': True}
+    enabled_config: CoreConfig = CoreConfig(validation_error_cause=True)
 
     def multi_raise_py_error(v: Any) -> Any:
         try:
@@ -546,7 +552,7 @@ def test_validation_error_cause_contents():
         s2.validate_python('anything')
 
     cause_group = exc_info.value.__cause__
-    assert isinstance(cause_group, BaseExceptionGroup)
+    assert isinstance(cause_group, BaseExceptionGroup)  # noqa: F821,RUF100  # gated on 3.11+ above
     assert len(cause_group.exceptions) == 1
 
     cause = cause_group.exceptions[0]
@@ -571,7 +577,7 @@ def test_validation_error_cause_contents():
     with pytest.raises(ValidationError) as exc_info:
         s3.validate_python('anything')
 
-    assert isinstance(exc_info.value.__cause__, BaseExceptionGroup)
+    assert isinstance(exc_info.value.__cause__, BaseExceptionGroup)  # noqa: F821,RUF100  # gated on 3.11+ above
     assert len(exc_info.value.__cause__.exceptions) == 1
     cause = exc_info.value.__cause__.exceptions[0]
     assert cause.__notes__ and cause.__notes__[-1].startswith('\nPydantic: ')
@@ -580,7 +586,7 @@ def test_validation_error_cause_contents():
     assert isinstance(subcause, ValidationError)
 
     cause_group = subcause.__cause__
-    assert isinstance(cause_group, BaseExceptionGroup)
+    assert isinstance(cause_group, BaseExceptionGroup)  # noqa: F821,RUF100  # gated on 3.11+ above
     assert len(cause_group.exceptions) == 1
 
     cause = cause_group.exceptions[0]
@@ -599,7 +605,7 @@ def test_validation_error_cause_contents():
 def test_validation_error_cause_contents_legacy():
     from exceptiongroup import BaseExceptionGroup
 
-    enabled_config: CoreConfig = {'validation_error_cause': True}
+    enabled_config: CoreConfig = CoreConfig(validation_error_cause=True)
 
     def multi_raise_py_error(v: Any) -> Any:
         try:
@@ -677,10 +683,10 @@ class CauseResult(enum.Enum):
     [  # Without the backport should still work after 3.10 as not needed:
         (
             'Enabled',
-            {'validation_error_cause': True},
+            CoreConfig(validation_error_cause=True),
             CauseResult.CAUSE if sys.version_info >= (3, 11) else CauseResult.IMPORT_ERROR,
         ),
-        ('Disabled specifically', {'validation_error_cause': False}, CauseResult.NO_CAUSE),
+        ('Disabled specifically', CoreConfig(validation_error_cause=False), CauseResult.NO_CAUSE),
         ('Disabled implicitly', {}, CauseResult.NO_CAUSE),
     ],
 )
@@ -715,7 +721,7 @@ def test_validation_error_cause_config_variants(desc: str, config: CoreConfig, e
 def test_validation_error_cause_traceback_preserved():
     """Makes sure historic bug of traceback being lost is fixed."""
 
-    enabled_config: CoreConfig = {'validation_error_cause': True}
+    enabled_config: CoreConfig = CoreConfig(validation_error_cause=True)
 
     def singular_raise_py_error(v: Any) -> Any:
         raise ValueError('Oh no!')
@@ -743,7 +749,7 @@ class BadRepr:
 
 
 def test_error_on_repr(pydantic_version):
-    s = SchemaValidator({'type': 'int'})
+    s = SchemaValidator(core_schema.int_schema())
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python(BadRepr())
 
@@ -769,7 +775,7 @@ def test_error_on_repr(pydantic_version):
 
 
 def test_error_json(pydantic_version):
-    s = SchemaValidator({'type': 'str', 'min_length': 3})
+    s = SchemaValidator(core_schema.str_schema(min_length=3))
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('12')
 
@@ -847,7 +853,7 @@ def test_error_json_python_error(pydantic_version: str):
 
 
 def test_error_json_cycle():
-    s = SchemaValidator({'type': 'str', 'min_length': 3})
+    s = SchemaValidator(core_schema.str_schema(min_length=3))
     cycle = []
     cycle.append(cycle)
     msg = '[type=string_type, input_value=[[...]], input_type=list]'
@@ -869,7 +875,7 @@ class CustomStr:
 
 
 def test_error_json_unknown():
-    s = SchemaValidator({'type': 'str'})
+    s = SchemaValidator(core_schema.str_schema())
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python(Foobar())
 
@@ -1083,7 +1089,7 @@ def test_loc_with_dots(pydantic_version):
 
 
 def test_hide_input_in_error() -> None:
-    s = SchemaValidator({'type': 'int'})
+    s = SchemaValidator(core_schema.int_schema())
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('definitely not an int')
 
@@ -1092,7 +1098,7 @@ def test_hide_input_in_error() -> None:
 
 
 def test_hide_input_in_json() -> None:
-    s = SchemaValidator({'type': 'int'})
+    s = SchemaValidator(core_schema.int_schema())
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('definitely not an int')
 
@@ -1105,7 +1111,7 @@ def test_hide_input_in_json() -> None:
     reason='PyPy before 3.9 cannot pickle this correctly',
 )
 def test_validation_error_pickle() -> None:
-    s = SchemaValidator({'type': 'int'})
+    s = SchemaValidator(core_schema.int_schema())
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('definitely not an int')
 
@@ -1116,7 +1122,7 @@ def test_validation_error_pickle() -> None:
 
 @pytest.mark.skipif('PYDANTIC_ERRORS_INCLUDE_URL' in os.environ, reason="can't test when envvar is set")
 def test_errors_include_url() -> None:
-    s = SchemaValidator({'type': 'int'})
+    s = SchemaValidator(core_schema.int_schema())
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('definitely not an int')
     assert 'https://errors.pydantic.dev' in repr(exc_info.value)
@@ -1143,7 +1149,7 @@ def test_errors_include_url_envvar(env_var, env_var_value, expected_to_have_url)
     Since it can only be set before `ValidationError.__repr__()` is first called,
     we need to spawn a subprocess to test it.
     """
-    code = "import pydantic_core; pydantic_core.SchemaValidator({'type': 'int'}).validate_python('ooo')"
+    code = "import pydantic_core; from pydantic_core import core_schema; pydantic_core.SchemaValidator(core_schema.int_schema()).validate_python('ooo')"
     env = os.environ.copy()
     env.pop('PYDANTIC_ERRORS_OMIT_URL', None)  # in case the ambient environment has it set
     if env_var_value is not None:
