@@ -1,13 +1,13 @@
 import json
 import platform
 import re
-from typing import List
 
 import pytest
 from dirty_equals import IsFloatNan, IsList
 
 import pydantic_core
 from pydantic_core import (
+    CoreConfig,
     PydanticSerializationError,
     SchemaSerializer,
     SchemaValidator,
@@ -26,28 +26,28 @@ from .conftest import Err
     [('false', False), ('true', True), ('0', False), ('1', True), ('"yes"', True), ('"no"', False)],
 )
 def test_bool(input_value, output_value):
-    v = SchemaValidator({'type': 'bool'})
+    v = SchemaValidator(core_schema.bool_schema())
     assert v.validate_json(input_value) == output_value
 
 
 @pytest.mark.parametrize('input_value', ['[1, 2, 3]', b'[1, 2, 3]', bytearray(b'[1, 2, 3]')])
 def test_input_types(input_value):
-    v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
+    v = SchemaValidator(core_schema.list_schema(items_schema=core_schema.int_schema()))
     assert v.validate_json(input_value) == [1, 2, 3]
 
 
 def test_input_type_invalid():
-    v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
+    v = SchemaValidator(core_schema.list_schema(items_schema=core_schema.int_schema()))
     with pytest.raises(ValidationError, match=r'JSON input should be string, bytes or bytearray \[type=json_type,'):
         v.validate_json([])
 
 
 def test_null():
-    assert SchemaValidator({'type': 'none'}).validate_json('null') is None
+    assert SchemaValidator(core_schema.none_schema()).validate_json('null') is None
 
 
 def test_str():
-    s = SchemaValidator({'type': 'str'})
+    s = SchemaValidator(core_schema.str_schema())
     assert s.validate_json('"foobar"') == 'foobar'
     with pytest.raises(ValidationError, match=r'Input should be a valid string \[type=string_type,'):
         s.validate_json('false')
@@ -56,7 +56,7 @@ def test_str():
 
 
 def test_bytes():
-    s = SchemaValidator({'type': 'bytes'})
+    s = SchemaValidator(core_schema.bytes_schema())
     assert s.validate_json('"foobar"') == b'foobar'
     with pytest.raises(ValidationError, match=r'Input should be a valid bytes \[type=bytes_type,'):
         s.validate_json('false')
@@ -82,7 +82,7 @@ _BIG_NUMBER_STR = '1' + ('0' * 40)
     ],
 )
 def test_int(input_value, expected):
-    v = SchemaValidator({'type': 'int'})
+    v = SchemaValidator(core_schema.int_schema())
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_json(input_value)
@@ -103,7 +103,7 @@ def test_int(input_value, expected):
     ],
 )
 def test_float(input_value, expected):
-    v = SchemaValidator({'type': 'float'})
+    v = SchemaValidator(core_schema.float_schema())
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_json(input_value)
@@ -113,13 +113,12 @@ def test_float(input_value, expected):
 
 def test_typed_dict():
     v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'fields': {
-                'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                'field_b': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-            },
-        }
+        core_schema.typed_dict_schema(
+            fields={
+                'field_a': core_schema.typed_dict_field(schema=core_schema.str_schema()),
+                'field_b': core_schema.typed_dict_field(schema=core_schema.int_schema()),
+            }
+        )
     )
 
     # language=json
@@ -132,20 +131,21 @@ def test_typed_dict():
 
 
 def test_float_no_remainder():
-    v = SchemaValidator({'type': 'int'})
+    v = SchemaValidator(core_schema.int_schema())
     assert v.validate_json('123.0') == 123
 
 
 def test_error_loc():
     v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'fields': {
-                'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'list', 'items_schema': {'type': 'int'}}}
+        core_schema.typed_dict_schema(
+            fields={
+                'field_a': core_schema.typed_dict_field(
+                    schema=core_schema.list_schema(items_schema=core_schema.int_schema())
+                )
             },
-            'extras_schema': {'type': 'int'},
-            'extra_behavior': 'allow',
-        }
+            extras_schema=core_schema.int_schema(),
+            extra_behavior='allow',
+        )
     )
 
     # assert v.validate_json('{"field_a": [1, 2, "3"]}') == ({'field_a': [1, 2, 3]}, {'field_a'})
@@ -163,7 +163,9 @@ def test_error_loc():
 
 
 def test_dict():
-    v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'int'}, 'values_schema': {'type': 'int'}})
+    v = SchemaValidator(
+        core_schema.dict_schema(keys_schema=core_schema.int_schema(), values_schema=core_schema.int_schema())
+    )
     assert v.validate_json('{"1": 2, "3": 4}') == {1: 2, 3: 4}
 
     # duplicate keys, the last value wins, like with python
@@ -172,12 +174,12 @@ def test_dict():
 
 
 def test_dict_any_value():
-    v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'str'}})
+    v = SchemaValidator(core_schema.dict_schema(keys_schema=core_schema.str_schema()))
     assert v.validate_json('{"1": 1, "2": "a", "3": null}') == {'1': 1, '2': 'a', '3': None}
 
 
 def test_json_invalid():
-    v = SchemaValidator({'type': 'bool'})
+    v = SchemaValidator(core_schema.bool_schema())
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json('"foobar')
@@ -249,7 +251,7 @@ def test_to_jsonable_python_fallback():
 
 def test_to_jsonable_python_schema_serializer():
     class Foobar:
-        def __init__(self, my_foo: int, my_inners: List['Foobar']):
+        def __init__(self, my_foo: int, my_inners: list['Foobar']):
             self.my_foo = my_foo
             self.my_inners = my_inners
 
@@ -280,9 +282,9 @@ def test_to_jsonable_python_schema_serializer():
     Foobar.__pydantic_serializer__ = s
 
     instance = Foobar(my_foo=1, my_inners=[Foobar(my_foo=2, my_inners=[])])
-    assert to_jsonable_python(instance) == {'myFoo': 1, 'myInners': [{'myFoo': 2, 'myInners': []}]}
+    assert to_jsonable_python(instance, by_alias=True) == {'myFoo': 1, 'myInners': [{'myFoo': 2, 'myInners': []}]}
     assert to_jsonable_python(instance, by_alias=False) == {'my_foo': 1, 'my_inners': [{'my_foo': 2, 'my_inners': []}]}
-    assert to_json(instance) == b'{"myFoo":1,"myInners":[{"myFoo":2,"myInners":[]}]}'
+    assert to_json(instance, by_alias=True) == b'{"myFoo":1,"myInners":[{"myFoo":2,"myInners":[]}]}'
     assert to_json(instance, by_alias=False) == b'{"my_foo":1,"my_inners":[{"my_foo":2,"my_inners":[]}]}'
 
 
@@ -384,7 +386,7 @@ def test_json_bytes_base64_round_trip():
     encoded_url = b'"2AfBVHgkkUYl8_NJythADO7Dq_9_083N-cIQ5KGwMWU="'
     assert to_json(data, bytes_mode='base64') == encoded_url
 
-    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'base64'})
+    v = SchemaValidator(core_schema.bytes_schema(), config=CoreConfig(val_json_bytes='base64'))
     assert v.validate_json(encoded_url) == data
     assert v.validate_json(encoded_std) == data
 
@@ -395,14 +397,20 @@ def test_json_bytes_base64_round_trip():
 
     assert to_json({'key': data}, bytes_mode='base64') == b'{"key":' + encoded_url + b'}'
     v = SchemaValidator(
-        {'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'bytes'}},
-        {'val_json_bytes': 'base64'},
+        core_schema.dict_schema(keys_schema=core_schema.str_schema(), values_schema=core_schema.bytes_schema()),
+        config=CoreConfig(val_json_bytes='base64'),
     )
     assert v.validate_json(b'{"key":' + encoded_url + b'}') == {'key': data}
 
 
+def test_json_bytes_base64_no_padding():
+    v = SchemaValidator(core_schema.bytes_schema(), config=CoreConfig(val_json_bytes='base64'))
+    base_64_without_padding = 'bm8tcGFkZGluZw'
+    assert v.validate_json(json.dumps(base_64_without_padding)) == b'no-padding'
+
+
 def test_json_bytes_base64_invalid():
-    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'base64'})
+    v = SchemaValidator(core_schema.bytes_schema(), config=CoreConfig(val_json_bytes='base64'))
     wrong_input = 'wrong!'
     with pytest.raises(ValidationError) as exc_info:
         v.validate_json(json.dumps(wrong_input))
@@ -410,7 +418,7 @@ def test_json_bytes_base64_invalid():
         {
             'type': 'bytes_invalid_encoding',
             'loc': (),
-            'msg': f'Data should be valid base64: Invalid symbol {ord("!")}, offset {len(wrong_input)-1}.',
+            'msg': f'Data should be valid base64: Invalid symbol {ord("!")}, offset {len(wrong_input) - 1}.',
             'input': wrong_input,
         }
     ]
@@ -421,19 +429,19 @@ def test_json_bytes_hex_round_trip():
     encoded = b'"68656c6c6f"'
     assert to_json(data, bytes_mode='hex') == encoded
 
-    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'hex'})
+    v = SchemaValidator(core_schema.bytes_schema(), config=CoreConfig(val_json_bytes='hex'))
     assert v.validate_json(encoded) == data
 
     assert to_json({'key': data}, bytes_mode='hex') == b'{"key":"68656c6c6f"}'
     v = SchemaValidator(
-        {'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'bytes'}},
-        {'val_json_bytes': 'hex'},
+        core_schema.dict_schema(keys_schema=core_schema.str_schema(), values_schema=core_schema.bytes_schema()),
+        config=CoreConfig(val_json_bytes='hex'),
     )
     assert v.validate_json('{"key":"68656c6c6f"}') == {'key': data}
 
 
 def test_json_bytes_hex_invalid():
-    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'hex'})
+    v = SchemaValidator(core_schema.bytes_schema(), config=CoreConfig(val_json_bytes='hex'))
 
     wrong_input = 'a'
     with pytest.raises(ValidationError) as exc_info:
